%*****************Analytical forward model accounting for river discharge**
%*****************including the effect of residual water level slope*******
%*****************Output:eta_Qf---->Tidal amplitude (m)*************************
%                         h_Qf---->Final mean depth (m)
%                         v_Qf----->Tidal velocity amplitude (m/s)
%                         c_Qf----->wave celerity (m/s)
%                         mu_Qf---->velocity number (-)
%                         delta_Qf--->damping number (-)
%                         lambda_Qf--->celerity number (-)
%                         epsilon_Qf--->phase lag
%Reference: (1)Cai, H., Savenije, H. H. G., and Toffolon, M.:
%              Linking the river to the estuary: influence of river discharge on tidal damping,
%              Hydrol. Earth Syst. Sci., 18, 287-304, doi:10.5194/hess-18-287-2014, 2014.
%           (2)Cai, H., Savenije, H. H. G., and Jiang, C.: 
%              Analytical approach for predicting fresh water discharge in an estuary based on tidal water level observations, 
%              Hydrol. Earth Syst. Sci. Discuss., 11, 7053-7087, doi:10.5194/hessd-11-7053-2014, 2014.
%Author: Huayang Cai (h.cai@tudelft.nl)
%Date:03/09/2014
%**************************************************************************
clc,clear all
close all
L=1200; %length of the estuary
dx=1000; %distance interval
x=0:dx:L*1000; 
T=12.42*3600;%tidal period
w=2*pi/T; %tidal frequency
g=9.81; %gravity acceleration
eta0=2; %tidal amplitude at the estuary mouth
eta(1)=eta0;
%*****************Input of river discharge*********************************
Q=5000; %river discharge imposed at the landward boundary
%*****************Input of estuarine geometry******************************
B0=5000;  % the width at the estuary mouth
B_min=300; % the vanishing width when distance approaches infinity
a1=200*1000; %convergence length
K1=45; %Manning-Strickler friction coefficient
rs1=1; %Storage width ratio
h0=ones(length(x),1).*10;
h=h0'; %horizontal bed
       %**********************************************
       for i=1:length(x)
        rs(i)=rs1;
        Ks(i)=K1;
        Qf(i)=Q;
        c0(i)=sqrt((g*h(i))/rs(i)); % the classical wave celerity
        gamma(i)=c0(i)*(B0-B_min)*exp(-x(i)/a1)/(a1*w*(B_min+(B0-B_min)*exp(-x(i)/a1))); %estuary shape number
        B(i)=B_min+(B0-B_min)*exp(-(x(i)/a1)); % width
        A(i)=B(i)*h(i); % cross-sectional area
        vdis(i)=Qf(i)/A(i); % fresh water flow velocity
       end
%*****************end of input of the geometry*****************************
%*****************start of the model calculation***************************
zeta(1)=eta(1)/h(1); % the tidal amplitude-to-depth ratio
f(1)=g/(Ks(1)^2*h(1)^(1/3))/(1-(1.33*zeta(1))^2); %dimensionless friction factor
chi(1)=rs(1)*f(1)*c0(1)*zeta(1)/(w*h(1)); % friction number
[mu(1),delta(1),lambda(1),epsilon(1)]=f_new_2012(gamma(1),chi(1)); %Analytical model without considering river discharge
Eta_Dx(1)=delta(1)*eta(1)*w/c0(1); % the rate of tidal damping
v(1)=eta(1)*c0(1)*rs(1)*mu(1)/h(1); % the real scale of velocity
c(1)=c0(1)/lambda(1); % the wave celerity
%****************************************************
for i=1:length(x)-1
       eta(i+1)=eta(i)+Eta_Dx(i)*dx; %Integration of the damping number
       zeta(i+1)=eta(i+1)/h(i+1); % the tidal amplitude-to-depth ratio
       f(i+1)=g/(Ks(i+1)^2*h(i+1)^(1/3))/(1-(1.33*zeta(i+1))^2);
       chi(i+1)=rs(i+1)*f(i+1)*c0(i+1)*zeta(i+1)/(w*h(i+1));
       [mu(i+1),delta(i+1),lambda(i+1),epsilon(i+1)]=f_new_2012(gamma(i+1),chi(i+1));     
       Eta_Dx(i+1)=delta(i+1)*eta(i+1)*w/c0(i+1); % the rate of tidal damping
       v(i+1)=eta(i+1)*c0(i+1)*rs(i+1)*mu(i+1)/h(i+1); % the real scale of velocity
       c(i+1)=c0(i+1)/lambda(i+1);% the wave celerity
end
%************* The influence of river discharge on tidal damping**********
mu_zero=mu;
lambda_zero=lambda;
delta_zero=delta;
epsilon_zero=epsilon;
mu_Qf=mu;
lambda_Qf=lambda;
delta_Qf=delta;
epsilon_Qf=epsilon;
sum_error=1;
kk=0;
while (sum_error>0.01 & kk<50)
kk=kk+1;
eta_Qf(1)=eta0;
zeta(1)=eta_Qf(1)/h(1); 
%*********************************************************
f(1)=(g/(Ks(1)^2*h(1)^(1/3)))/(1-(1.33*zeta(1))^2); % the dimensionless friction factor
Xi(1)=(rs(1)*f(1)*c0(1)*eta_Qf(1))/(w*h(1)^2); % the friction number
alpha(1)=Qf(1)/(A(1)*v(1));
x0=[mu_Qf(1), delta_Qf(1), lambda_Qf(1)]';
uf_v(1)=Qf(1)/(A(1)*v(1));
if(uf_v(1)==0)
    L0(1)=0;L1(1)=16/(3*pi);
elseif(uf_v(1)<=1)
    ga=acos(-uf_v(1));
L0(1)=(2+cos(2*ga))*(2-4*ga/pi)+6*sin(2*ga)/pi;
L1(1)=6*sin(ga)/pi+2*sin(3*ga)/(3*pi)+(4-8*ga/pi)*cos(ga);
else
L0(1)=-2-4*uf_v(1)^2;
    L1(1)=4*uf_v(1);
end
if(alpha(1)<=(mu_Qf(1)*lambda_Qf(1)))
[y,iter]=findzero_new_discharge_tide(gamma(1), Xi(1), rs(1),alpha(1),zeta(1),x0);
else
    [y,iter]=findzero_new_discharge_river(gamma(1), Xi(1), rs(1),alpha(1),zeta(1),x0);
end
mu_Qf(1)=y(1);
delta_Qf(1)=y(2);
lambda_Qf(1)=y(3);
epsilon_Qf(1)=atan(y(3)/(gamma(1)-y(2)));
v_Qf(1)=eta_Qf(1)*c0(1)*rs(1)*mu_Qf(1)/h(1); % the real scale of velocity
phi(1)=vdis(1)/(v_Qf(1)*sin(epsilon_Qf(1))); % the ratio between river and tide
Eta_Dxf(1)=delta_Qf(1)*eta_Qf(1)*w/c0(1);
c_Qf(1)=c0(1)/lambda_Qf(1);
%%
if(v_Qf(1)*sin(epsilon_Qf(1))>vdis(1))
Fs1(1)=1/3*(v_Qf(1)*sin(epsilon_Qf(1))-vdis(1))^2*(1/(Ks(1)^2*(h(1)+eta_Qf(1))^(4/3)));
else
Fs1(1)=-1/3*(v_Qf(1)*sin(epsilon_Qf(1))-vdis(1))^2*(1/(Ks(1)^2*(h(1)+eta_Qf(1))^(4/3)));
end
Fs(1)=Fs1(1)-1/3*(-v_Qf(1)*sin(epsilon_Qf(1))-vdis(1))^2*(1/(Ks(1)^2*(h(1)-eta_Qf(1))^(4/3)));
Fl(1)=1/6*(L0(1)*v_Qf(1)^2/4+L1(1)*v_Qf(1)*v_Qf(1)*sin(epsilon_Qf(1))/2)*(1/(Ks(1)^2*(h(1)+eta_Qf(1))^(4/3)));
Fl(1)=Fl(1)+1/6*(L0(1)*v_Qf(1)^2/4-L1(1)*v_Qf(1)*v_Qf(1)*sin(epsilon_Qf(1))/2)*(1/(Ks(1)^2*(h(1)-eta_Qf(1))^(4/3)));
f_av(1)=Fs(1)+Fl(1);
h_Qf(1)=h0(1);
c0_Qf(1)=sqrt(g*h_Qf(1)/rs(1));
gamma_Qf(1)=c0_Qf(1)*(B0-B_min)*exp(-x(1)/a1)/(a1*w*(B_min+(B0-B_min)*exp(-x(1)/a1)));
 A_Qf(1)=B(1)*h_Qf(1);
 vdis_Qf(1)=Qf(1)/A_Qf(1);
%****************start to loop*********************************************
for i=1:length(x)-1
        eta_Qf(i+1)=eta_Qf(i)+Eta_Dxf(i)*dx; %Integration of the damping number
        zeta(i+1)=eta_Qf(i+1)/h(i+1); %the dimensionless amplitude
        %*********************************      
f(i+1)=(g/(Ks(i+1)^2*h(i+1)^(1/3)))/(1-(1.33*zeta(i+1))^2); % the dimensionless friction factor
Xi(i+1)=(rs(i+1)*f(i+1)*c0(i+1)*eta_Qf(i+1))/(w*h(i+1)^2); % the friction number
alpha(i+1)=Qf(i+1)/(A(i+1)*v(i+1));
x0=[mu_Qf(i+1), delta_Qf(i+1), lambda_Qf(i+1)]';
uf_v(i+1)=Qf(i+1)/(A(i+1)*v(i+1));
if(uf_v(i+1)==0)
    L0(i+1)=0;L1(i+1)=16/(3*pi);
elseif(uf_v(i+1)<=1)
    ga=acos(-uf_v(i+1));
L0(i+1)=(2+cos(2*ga))*(2-4*ga/pi)+6*sin(2*ga)/pi;
L1(i+1)=6*sin(ga)/pi+2*sin(3*ga)/(3*pi)+(4-8*ga/pi)*cos(ga);
else
    L0(i+1)=-2-4*uf_v(i+1)^2;
    L1(i+1)=4*uf_v(i+1);
end
if(alpha(i+1)<=(mu_Qf(i+1)*lambda_Qf(i+1)))
[y,iter]=findzero_new_discharge_tide(gamma(i+1), Xi(i+1), rs(i+1),alpha(i+1),zeta(i+1),x0);
else
    [y,iter]=findzero_new_discharge_river(gamma(i+1), Xi(i+1), rs(i+1),alpha(i+1),zeta(i+1),x0);
end
mu_Qf(i+1)=y(1);
delta_Qf(i+1)=y(2);
lambda_Qf(i+1)=y(3);
epsilon_Qf(i+1)=atan(y(3)/(gamma(i+1)-y(2)));
v_Qf(i+1)=eta_Qf(i+1)*c0(i+1)*rs(i+1)*mu_Qf(i+1)/h(i+1); % the real scale of velocity
phi(i+1)=vdis(i+1)/(v_Qf(i+1)*sin(epsilon_Qf(i+1))); % the ratio between river and tide
Eta_Dxf(i+1)=delta_Qf(i+1)*eta_Qf(i+1)*w/c0(i+1);
c_Qf(i+1)=c0(i+1)/lambda_Qf(i+1);
if((v_Qf(i+1)*sin(epsilon_Qf(i+1)))>=vdis(i+1))
Fs1(i+1)=2/3*((v_Qf(i+1)*sin(epsilon_Qf(i+1))-vdis(i+1))^2*(1/(Ks(i+1)^2*(h(i+1)+eta_Qf(i+1))^(4/3))));
else
Fs1(i+1)=-2/3*((v_Qf(i+1)*sin(epsilon_Qf(i+1))-vdis(i+1))^2*(1/(Ks(i+1)^2*(h(i+1)+eta_Qf(i+1))^(4/3))));
end
Fs(i+1)=Fs1(i+1)-2/3*((v_Qf(i+1)*sin(epsilon_Qf(i+1))+vdis(i+1))^2*(1/(Ks(i+1)^2*(h(i+1)-eta_Qf(i+1))^(4/3))));
Fl(i+1)=1/3*((L0(i+1)*v_Qf(i+1)^2/4+L1(i+1)*v_Qf(i+1)*v_Qf(i+1)*sin(epsilon_Qf(i+1))/2)*(1/(Ks(i+1)^2*(h(i+1)+eta_Qf(i+1))^(4/3))));
Fl(i+1)=Fl(i+1)+1/3*((L0(i+1)*v_Qf(i+1)^2/4-L1(i+1)*v_Qf(i+1)*v_Qf(i+1)*sin(epsilon_Qf(i+1))/2)*(1/(Ks(i+1)^2*(h(i+1)-eta_Qf(i+1))^(4/3))));
f_av(i+1)=1/2*(Fs(i+1)+Fl(i+1));
h_Qf(i+1)=-sum(f_av(1:i).*dx)+h0(i+1);
c0_Qf(i+1)=sqrt(g*h_Qf(i+1)/rs(i+1));
gamma_Qf(i+1)=c0_Qf(i+1)*(B0-B_min)*exp(-x(i+1)/a1)/(a1*w*(B_min+(B0-B_min)*exp(-x(i+1)/a1)));
 A_Qf(i+1)=B(i+1)*h_Qf(i+1);
 vdis_Qf(i+1)=Qf(i+1)/A_Qf(i+1);
end
gamma_z=c0_Qf./w.*gradient(h_Qf,dx)./h_Qf;
sum_error=sum(abs(lambda_zero-lambda_Qf)+abs(mu_zero-mu_Qf)+abs(delta_zero-delta_Qf))
    lambda_zero=lambda_Qf;
    mu_zero=mu_Qf;
    delta_zero=delta_Qf;
    v=sqrt(v_Qf./v).*v_Qf; 
    h=sqrt(h_Qf./h).*h_Qf;
    c0=sqrt(c0_Qf./c0).*c0_Qf;
    gamma=gamma_Qf-gamma_z;
    A=A_Qf;
    vdis=vdis_Qf;
    if(sum_error<0.01)
        kk2=kk; %number of iteration to satisfy tolerance
        disp(['Number of iteration to satisfy tolerance ',num2str(kk2)])
        break
    end
end
%%
%****************************************Plot******************************
figure
subplot(211)
plot(x,eta_Qf,'-b') %longitudinal variation of tidal amplitude
grid on
ylabel('Tidal amplitude \it\eta\rm (m)')
title(['Q_f=',num2str(Q),' m^3s^{-1}, K=',num2str(K1),' m^{1/3}s^{-1}, a='...
    num2str(a1/1000),' km, h_0=',num2str(mean(h0)),' m'])
%*******************
subplot(212)
plot(x,h0,'-b',x,h,'--r') %%longitudinal variation of depth
grid on
xlabel('Distance from the estuary mouth (m)')
ylabel('Depth $$\overline{h}$$ (m)','interpreter','latex')
legend('Imposed mean depth','Final mean depth','location','Northwest')